/**
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package com.sintecmedia.components.renderkit;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.component.ValueHolder;
import javax.faces.component.html.HtmlCommandButton;
import javax.faces.context.FacesContext;

import org.ajax4jsf.context.AjaxContext;
import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSFunctionDefinition;
import org.ajax4jsf.javascript.JSLiteral;
import org.ajax4jsf.renderkit.AjaxComponentRendererBase;
import org.ajax4jsf.renderkit.AjaxRendererUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sintecmedia.components.component.UIWeekDaysButtons;
import com.sintecmedia.components.event.DaySelectedEvent;

/**
 * @author asmirnov@exadel.com (latest modification by $Author: igor $)
 * @version $Revision: 1872 $ $Date: 2010-05-06 11:34:42 +0300 (Thu, 06 May 2010) $
 * 
 */
public abstract class WeekDaysButtonsRendererBase 
		extends AjaxComponentRendererBase {

	private static final Log _log = 
		LogFactory.getLog( WeekDaysButtonsRendererBase.class );
	
	protected void doDecode(FacesContext context, UIComponent component) {
		Map paramMap = getParamMap(context);
    	String clientId = component.getClientId(context);
    	String param = (String) paramMap.get(clientId);        
        if (param == null) {
        	return;
        }
        
		UIWeekDaysButtons dayButtons = (UIWeekDaysButtons) component;
		int newDay = dayButtons.getDayIndex();
		int day = dayButtons.getCurrentDayIndex();
		if( newDay != 0 && newDay != day ) {
			DaySelectedEvent event =
				new DaySelectedEvent(			
						dayButtons, String.valueOf(day), param, newDay );
		     event.queue();
		}        
	}

	private Map getParamMap(FacesContext context) {
        return context.getExternalContext().getRequestParameterMap();
    }

	public String getOnClick(
			FacesContext context, UIComponent component, Integer dayIndex ) {
		return 
			"Event.fire(this, 'rich:weekdaysbuttons:ondayselect', {'dayIndex': '" + dayIndex + "'});";	
	}

	public Integer getIndexByDay( UIComponent component, Object dayName ) {
		return
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH 
				).get( dayName.toString() );
	}
	
	public String getSubmitFunction(
			FacesContext context, UIComponent component ) {
		JSFunctionDefinition definition = new JSFunctionDefinition("event");
	        
		JSFunction function = 
			AjaxRendererUtils.buildAjaxFunction( component, context );
        Map eventOptions = 
        	AjaxRendererUtils.buildEventOptions( context, component, true );
        Map parameters = (Map) eventOptions.get( "parameters" );
        
        Map params = getParameters( context,component );
        if(!params.isEmpty()){
        	parameters.putAll(params);
        }
        
        parameters.put(
        		component.getClientId(context), 
        		new JSLiteral("event.memo.page") );
        
        function.addParameter(eventOptions);
        StringBuffer buffer = new StringBuffer();
        function.appendScript(buffer);
        buffer.append("; return false;");

		String onDayChange = (String) component.getAttributes().get("ondayselect");
		if (onDayChange != null && onDayChange.length() != 0) {
		    JSFunctionDefinition onDayChangeDef = new JSFunctionDefinition("event");
		    onDayChangeDef.addToBody(onDayChange);
		    onDayChangeDef.addToBody("; return true;");

		    definition.addToBody("if (");
		    definition.addToBody(onDayChangeDef.toScript());
		    definition.addToBody("(event)) {");
		    definition.addToBody(buffer.toString());
		    definition.addToBody("}");
		} else {
		    definition.addToBody(buffer.toString());
		}
	        
		return definition.toScript();
	}
	
	//get UIParameter's Map
    protected Map getParameters(FacesContext context, UIComponent component){
    	Map parameters = new HashMap(); 
    	
    	if( component instanceof UIWeekDaysButtons ){
    		UIWeekDaysButtons weekDaysButtons = (UIWeekDaysButtons)component;
    		List children = weekDaysButtons.getChildren();
    		for (Iterator iterator = children.iterator(); iterator.hasNext();) {
				UIComponent child = (UIComponent) iterator.next();
				if(child instanceof UIParameter) {
					UIParameter param = (UIParameter)child;
					String name = param.getName();
					if (name != null) {
						parameters.put(name, param.getValue());
					}
				}
			}
    	}
    	    	
    	return parameters;
    }
    
	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		renderChildren(context, component);
	}

	public Object getValue(UIComponent uiComponent) {
		if (uiComponent instanceof ValueHolder) {
			return ((ValueHolder) uiComponent).getValue();
		}
		return uiComponent.getAttributes().get("value");
	}

	public String getType(UIComponent uiComponent) {
		String type;
		if (uiComponent instanceof HtmlCommandButton) {
			type = ((HtmlCommandButton) uiComponent).getType();
		} else {
			type = (String) uiComponent.getAttributes().get("type");
		}
		if (type == null) {
			type = "button";
		}
		return type;
	}

	protected boolean isSubmitted(FacesContext facesContext,
			UIComponent uiComponent) {
		// Componet accept only ajax requests.
		if (!AjaxContext.getCurrentInstance(facesContext).isAjaxRequest()) {
			return false;
		}
		if (getUtils().isBooleanAttribute(uiComponent, "disabled")) {
			return false;
		}
		String clientId = uiComponent.getClientId(facesContext);
		Map<String, String> paramMap = facesContext.getExternalContext()
				.getRequestParameterMap();
		Object value = paramMap.get(clientId);
		boolean submitted = null != value;
		if (submitted && _log.isDebugEnabled()) {
			_log.debug("Decode submit of the Ajax component " + clientId);
		}
		return submitted;
	}
	
	
	/**
	 * 
	 * @param component UIWeeklyCalendar component used.
	 * @return list of days from component data model as it called in default java.util.Calendar intance
	 */
	public List<String> getDaysOfWeek( UIComponent component ) {
		//Days map returned from calendar relays on day's name as key, so it
		//needs refinement by day's index.
		Map<String, Integer> daysMap = 
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH );
		String[] names = new String[ daysMap.size() ]; 
		for( String name : daysMap.keySet() ) {
			names[ daysMap.get( name ) - 1 ] = name;
		}			
		
		List<String> result = new ArrayList<String>( 7 );
		for( int i = 0; i < 7; i++ ) {			
			result.add( names[i] );
		}
		return result;
	}

	
	public String getFunctionChild ( FacesContext facesContext, UIComponent component) {
		System.out.println( "getFunctionChild");
		System.out.println( component.getChildCount());		
	return "";
	}
}
